@@ -19,11 +19,15 @@
 #include <stddef.h>
 #include <setjmp.h>
 #include <signal.h>
+#include <libc-internal.h>
+
+libc_hidden_proto(sigprocmask)
 
 /* This function is called by the `sigsetjmp' macro
    before doing a `__setjmp' on ENV[0].__jmpbuf.
    Always return zero.  */
 
+int __sigjmp_save (sigjmp_buf env, int savemask) attribute_hidden;
 int __sigjmp_save (sigjmp_buf env, int savemask)
 {
     env[0].__mask_was_saved = (savemask && 
